# Voice alerts for DeviationTx
This is a proof of concept build for a modification to allow
deviationTx to play arbitrary audio files for it's various alerts. It
requires an external audio player module to store and play the files,
which must be connected to a speaker. The audio module is connected to
the transmitter via the PPM signal interface, an optional hardware
reset signal, and whatever power it requires. These connections mean
an external audio module can be connected via the trainer port without
opening the transmitter.

Since the PPM port is used to communicate with the audio module, voice
alerts cannot be used if your model uses the PPM protocol, or uses PPM
input channels. Since development builds output to this serial device,
it is disabled for them as well, and just prints debugging
information.

The reset signal isn't required, though that is still being
tested. The best option is to use either the TMS or TCK pins that are
also used with the RF modules. If those aren't available, the buzzer
connections can be used **(or can it? We need to play everything in
that as well)**, though that may cause problems if the voice playback
is disabled for PPM input, the PPM protocol or development
builds. This also needs more testing.

Because it is a proof of concept, all it does is play a fixed sound in
place of the various alerts - there's no audio configuration
yet. That's the next step!

# Transmitter setup
You need to connect the audio playback board to your transmitter, and
then enable it in `hardware.ini`.

## Hardware
For now, there only two lines from the transmitter are used. You'll
want to find the `Transmit` and `Ground` lines on the trainer port. If
you have a trainer cable modded to get serial output from
development/debug builds, that will work fine if you connect `Ground`
to `Ground` and the `Transmit` line to the `Receive` pin on your audio
board.

## Configuration
Being a hardware mod, this is configure in `hardware.ini`. Set it in
the top section, where the extra switches are. The setting is
`extended-audio`, and the possible values are `audiofx` (for the
Adafruit audiofx board) and `dfplayer` (for the DFPlayer mini).

# Audio files
The audio files are played instead of the beeps that the built in
hardware provides. The file format and storage layout all depend on
the module being used. See the modules section below.

For now, the audio module support is allowed to be finicky and
painful, as the current system is temporary while we work out the user
interface for dealing with this new ability. The current sound system
wasn't really designed for the user configuration that the ability to
play audio files brings with it. I believe we want to let the user
select from different sounds, which clashes with the current sound
system. And of course, the result for stock hardware must fit on a 7e.

There are a number of free tools on the web for create audio files
from text. I used
[IBM's online demo](https://text-to-speech-demo.mybluemix.net/) to
create mine.

The alert numbers are currently:

 1. Transmitter power on
 2. Transmitter shutting down
 3. Volume test.
 4. Timer #1 expired
 5. Timer #2 expired
 6. Timer #3 expired
 7. Timer #4 expired
 8. Transmitter battery low
 9. Model binding
 10. One of the timers is expiring
 11. Key press beep (not done as a voice alert)
 12. Model saved.
 13. No room for text input beep (not done as a voice alert)
 14. First telemetry alarm (model battery in my files)
 15. Second telemetry alarm (radio signal strength in my files)
 16. Third telemetry alarm (model temperature in my files)
 17. Fourth telemetry alarm
 18. Fifth telemetry alarm
 19. Sixth telemetry alarm

# Audio modules
The audio module must have a TTL level serial control interface. Since
the transmitters don't have amplifiers, the audio module must provide
that, or provide appropriate outputs for whatever you are going to be
listening on.

## [Adafruit Audio FX Board](http://www.adafruit.com/products/2133)
There are a number of variations on this board. The one linked two
would be suitable for an external box with a cable to connect to the
trainer port and a headphone jack to play audio. Set `extended-audio`
in `hardware.ini` to `audiofx`.

### Hardware connections
You'll need a 3.3 or 5 volt power supply tied to `VIN` as well as
`Ground`. The three relevant pins on the board are labeled `UG`, `RX` and
`TX`, in that order. `UG` needs to be connected to `Ground` to enable
serial communications. `RX` and should be connected to the the trainer
data out (transmit) line. `TX` isn't used yet.

### Audio files
Audio files from the IBM text to speech tool have need to be
downsampled to 16000 samples per second for this board. The files must
be copied the board in alert order. The `#` command is used along with
the alert number to play the file, so the `L` command needs to list
the files in alert order as above. The two alerts that we don't play
voice for need empty files so that the numbering works out.

## [DFPlayer Mini](http://www.banggood.com/search/dfplayer-mini.html)
This is an small but very flexible MP3 player from the DF Robotics
folks. You'll need an SD card to hold audio files, which could well be
an advantage. No simple button setup, the serial protocol is binary,
not ascii, and it in general just isn't as friendly as the Adafruit
board. But none of that should matter once we get it working. Set
`extended-audio` in `hardware.ini` to `dfplayer`.

### Hardware connections
This board requires a 5 volt power supply. At least, mine buzzes
horribly if you only provide it with 3.3 volts. You will also need a
`Ground` line and the transmitter PPM output line connected to the `RX`
pin.

The three pins on my board aren't labeled, but `VCC` is in the corner
near the words `DFPLAYER` and `RX` is next to it. `TX` is next to that
as well. `GND` is the next-to-last pin on the same sound, and the two
speaker connections are on either side of it. No other connections are
needed.

### Audio files
These requires mp3 files in a variety of formats. I used 160 kpbs,
44.1Khz Stereo, as that's what the voice synthesizer software
generated. If you're tight for space, you can probably switch to mono
and much lower quality.

They need to be in the folder `MP3` on the SD card mounted on the
module. The names start with four digits. Alert number `N` will play
the file that starts with the value of n - 1. So the file
`0000welc.ogg` in the attached zip will play for alert 0, which is the
power on alert.

## [Mini sound module](http://www.banggood.com/search/mp3-sound-module.html)
Is used in TH9X open source firmware systems. It should work fine, but
the miniplayer seems like a better choice, so I have no plans on
supporting this.

## Your smart phone
The ability to connect to the trainer port makes the possibility of
using a portable device with the appropriate app and an OTG serial
cable an interesting possibility.

# Speakers
The buzzer built into the devo transmitters can't play audio even if
connected to an amplifier. If you're using an internal module, the
easy option is to replace the buzzer with a real speaker. The power
requirements will depend on the audio module. The area it fits in
on my devo 10 is roughly 25mm×25mm. I haven't found a good option yet;
suggestions welcome!

# Transmitter Connections
All my testing has been done with an external module connected via the
trainer port and an external power supply. For an internally installed
module, you'll find the `Ground`, `Transmit` and `Receive` lines at
on the inside of the trainer port. For `VCC`, you can use the same
points as you would for an RF module - 3.3 volts from a number of
places, and 5 volts from one of the CYRF pins. See the wiki
documentation about adding modules for more information on those.

Please note that in order to allow configuration, we'll need 2-way
communication with the audio module. I suspect the current odd
connection - plugging in a trainer cable shorts the serial `Transmit`
and `Receive` lines - will have to be changed. This could well break
trainer mode support on devention, as I suspect that short is how
devention detects that there's a plug inserted.
